==> gtoonz-0.4.2.tar.gz: gtoonz-0.4.2/src/download.c <==
#include <config.h>

#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "gtoonz.h"

static void
url_copy_cb (GtkWidget *widget,
	     gpointer data)
{
	Item *item = (Item *)data;
	GdkAtom atom;
	GtkClipboard *clipboard;

	atom = gdk_atom_intern ("CLIPBOARD", FALSE);
	clipboard = gtk_clipboard_get (atom);
	gtk_clipboard_set_text (clipboard, item->url, strlen(item->url));
}

static gboolean
button_press_cb (GtkWidget *widget,
		 GdkEventButton *event,
		 gpointer data)
{
	GtkWidget *menu;
	GtkWidget *button;

	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
		menu = gtk_menu_new ();

		button = gtk_menu_item_new_with_label (_("url copy"));
		g_signal_connect (G_OBJECT(button), "activate",
				  G_CALLBACK(url_copy_cb), data);
		gtk_menu_append (GTK_MENU(menu), button);
		gtk_widget_show (button);

		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
				event->button, event->time);
	}
	return TRUE;
}

static void
progressive_prepared_cb (GdkPixbufLoader *loader,
			 gpointer data)
{
	GdkPixbuf *pixbuf;
	GtkWidget *image = data;

	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
	gdk_pixbuf_fill (pixbuf, 0xaaaaaaff);
	gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
	return;
}

static void
progressive_update_cb (GdkPixbufLoader *loader,
		       gint x,
		       gint y,
		       gint width,
		       gint height,
		       gpointer data)
{
	GtkWidget *image = data;

	gtk_widget_queue_draw (image);
	return;
}

static void
close_cb (GnomeVFSAsyncHandle *handle,
	  GnomeVFSResult result,
	  gpointer data)
{
	if (data) ((Item *)data)->handle = 0;
	return;
}

static void
read_cb (GnomeVFSAsyncHandle *handle,
	 GnomeVFSResult result,
	 gpointer buffer,
	 GnomeVFSFileSize bytes_requested,
	 GnomeVFSFileSize bytes_read,
	 gpointer data)
{
	Item *item = (Item *)data;
	assert (item->handle == handle);

	if (result != GNOME_VFS_OK) {
		if (result == GNOME_VFS_ERROR_EOF) {
			status_update (_("Done"));

		} else {
			status_update (_("Download failed"));
		}
		gnome_vfs_async_close (handle, close_cb, item);
	} else {
		gdk_threads_enter ();
		gdk_pixbuf_loader_write (item->loader, buffer, bytes_read, NULL);
		gdk_threads_leave ();
		gnome_vfs_async_read (handle, buffer, BUF_SIZE,
				      (GnomeVFSAsyncReadCallback)read_cb, item);
	}
	return;
}

static void
open_cb (GnomeVFSAsyncHandle *handle,
	 GnomeVFSResult result,
	 gpointer data)
{
	Item *item = (Item *)data;
	
	if (result != GNOME_VFS_OK) {
		status_update (_("file not found"));
		item->handle = 0;
	} else {
		gnome_vfs_async_read (handle, item->buf, BUF_SIZE,
				      (GnomeVFSAsyncReadCallback)read_cb, item);
	}
	return;
}

void
download (Cartoon *cartoon)
{
	GSList *list = cartoon->item_list;
	Item *item;
	GtkWidget *image;
	
	status_update (_("Downloading..."));
	while (list) {
		item = list->data;

		item->event = gtk_event_box_new ();
		image = gtk_image_new_from_pixbuf (NULL);
		gtk_container_add (GTK_CONTAINER(item->event), image);
		gtk_box_pack_start (GTK_BOX(cartoon->images_box), item->event,
				    TRUE, TRUE, 0);
		g_signal_connect (G_OBJECT(item->event), "button-press-event",
				  G_CALLBACK(button_press_cb), item);
		gtk_widget_show_all (item->event);

		item->buf = g_new(char, BUF_SIZE);
		item->loader = gdk_pixbuf_loader_new ();
		g_signal_connect (G_OBJECT(item->loader), "area_prepared",
				  G_CALLBACK(progressive_prepared_cb), image);
		g_signal_connect (G_OBJECT(item->loader), "area_updated",
				  G_CALLBACK(progressive_update_cb), image);
		
		gnome_vfs_async_open(&item->handle, item->url, GNOME_VFS_OPEN_READ, -5,
				     (GnomeVFSAsyncOpenCallback)open_cb, item);
		list = list->next;
	}
	return;
}

==> gtoonz-0.4.4.tar.gz: gtoonz-0.4.4/src/download.c <==
#include <config.h>

#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "gtoonz.h"

static void
url_copy_cb (GtkWidget *widget,
	     gpointer data)
{
	Item *item = (Item *)data;
	GdkAtom atom;
	GtkClipboard *clipboard;

	atom = gdk_atom_intern ("CLIPBOARD", FALSE);
	clipboard = gtk_clipboard_get (atom);
	gtk_clipboard_set_text (clipboard, item->url, strlen(item->url));
}
/* file_*_cb ϵ忡 ϱ  Լ */
static void
file_close_cb (GnomeVFSAsyncHandle *handle,
	  GnomeVFSResult result,
	  gpointer data)
{
	return;
}
static void
file_read_cb (GnomeVFSAsyncHandle *handle,
	      GnomeVFSResult result,
	      gpointer buffer,
	      GnomeVFSFileSize bytes_requested,
	      GnomeVFSFileSize bytes_read,
	      gpointer data)
{
	LocalFile *file = (LocalFile *)data;
	GnomeVFSFileSize bytes;
	
	if (result != GNOME_VFS_OK) {
		if (result == GNOME_VFS_ERROR_EOF) {
			status_update (_("File download done"));
		} else {
			status_update (_("File download failed"));
		}
		gnome_vfs_close (file->fd);
		g_free (file->name);
		g_free (file->buf);
		g_free (file);
		gnome_vfs_async_close (handle, file_close_cb, NULL);
	} else {
		gdk_threads_enter ();
		gnome_vfs_write (file->fd, buffer, bytes_read, &bytes);
		gdk_threads_leave ();
		gnome_vfs_async_read (handle, buffer, BUF_SIZE,
				      (GnomeVFSAsyncReadCallback)file_read_cb, data);
	}
}
static void
file_open_cb (GnomeVFSAsyncHandle *handle,
	      GnomeVFSResult result,
	      gpointer data)
{
	LocalFile *file = (LocalFile *)data;
	
	gnome_vfs_async_read (handle, file->buf, BUF_SIZE,
			      (GnomeVFSAsyncReadCallback)file_read_cb, file);
}
static void
save_file_cb (GtkWidget *widget,
	      gpointer data)
{
	Item *item = (Item *)data;
	GnomeVFSAsyncHandle *handle;
	GnomeVFSResult result;
	LocalFile *file;

	file = g_new (LocalFile, 1);
	file->name = g_strdup_printf ("file:%s/%s",
				      g_get_home_dir (),
				      g_path_get_basename (item->url));
	result = gnome_vfs_create (&file->fd, file->name, GNOME_VFS_OPEN_WRITE, TRUE, 00600);
	if (result != GNOME_VFS_OK) {
		g_free (file->name);
		g_free (file);
		return;
	}
	file->buf = g_new0 (char, BUF_SIZE);

	status_update (_("File downloading... "));
	gnome_vfs_async_open(&handle, item->url,
			     GNOME_VFS_OPEN_READ, 0,
			     (GnomeVFSAsyncOpenCallback)file_open_cb, file);
}

static gboolean
button_press_cb (GtkWidget *widget,
		 GdkEventButton *event,
		 gpointer data)
{
	GtkWidget *menu;
	GtkWidget *button;

	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
		menu = gtk_menu_new ();

		button = gtk_menu_item_new_with_label (_("url copy"));
		g_signal_connect (G_OBJECT(button), "activate",
				  G_CALLBACK(url_copy_cb), data);
		gtk_menu_append (GTK_MENU(menu), button);
		gtk_widget_show (button);

		button = gtk_menu_item_new_with_label (_("save file"));
		g_signal_connect (G_OBJECT(button), "activate",
				  G_CALLBACK(save_file_cb), data);
		gtk_menu_append (GTK_MENU(menu), button);
		gtk_widget_show (button);

		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
				event->button, event->time);
	}
	return TRUE;
}

static void
progressive_prepared_cb (GdkPixbufLoader *loader,
			 gpointer data)
{
	GdkPixbuf *pixbuf;
	GtkWidget *image = data;

	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
	gdk_pixbuf_fill (pixbuf, 0xaaaaaaff);
	gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
	return;
}

static void
progressive_update_cb (GdkPixbufLoader *loader,
		       gint x,
		       gint y,
		       gint width,
		       gint height,
		       gpointer data)
{
	GtkWidget *image = data;

	gtk_widget_queue_draw (image);
	return;
}

static void
close_cb (GnomeVFSAsyncHandle *handle,
	  GnomeVFSResult result,
	  gpointer data)
{
	Item *item = (Item *)data;
	
	if (item) item->handle = 0;
	gdk_window_set_cursor (item->cartoon->app->window, 0);
	
	return;
}

static void
read_cb (GnomeVFSAsyncHandle *handle,
	 GnomeVFSResult result,
	 gpointer buffer,
	 GnomeVFSFileSize bytes_requested,
	 GnomeVFSFileSize bytes_read,
	 gpointer data)
{
	Item *item = (Item *)data;
	assert (item->handle == handle);

	if (result != GNOME_VFS_OK) {
		if (result == GNOME_VFS_ERROR_EOF) {
			status_update (_("Done"));
		} else {
			status_update (_("Download failed"));
		}
		gnome_vfs_async_close (handle, close_cb, item);
	} else {
		gdk_threads_enter ();
		gdk_pixbuf_loader_write (item->loader, buffer, bytes_read, NULL);
		gdk_threads_leave ();
		gnome_vfs_async_read (handle, buffer, BUF_SIZE,
				      (GnomeVFSAsyncReadCallback)read_cb, item);
	}
	return;
}

static void
open_cb (GnomeVFSAsyncHandle *handle,
	 GnomeVFSResult result,
	 gpointer data)
{
	Item *item = (Item *)data;
	
	if (result != GNOME_VFS_OK) {
		status_update (_("file not found"));
		item->handle = 0;
		gdk_window_set_cursor (item->cartoon->app->window, 0);
	} else {
		gnome_vfs_async_read (handle, item->buf, BUF_SIZE,
				      (GnomeVFSAsyncReadCallback)read_cb, item);
	}
	return;
}

void
download (Cartoon *cartoon)
{
	GSList *list = cartoon->item_list;
	Item *item;
	GtkWidget *image;
	GdkCursor *cursor;
	
	status_update (_("Downloading..."));
	cursor = gdk_cursor_new (GDK_WATCH);
	gdk_window_set_cursor (cartoon->app->window, cursor);
	gdk_cursor_unref (cursor);
	
	while (list) {
		item = list->data;

		item->cartoon = cartoon;
		item->event = gtk_event_box_new ();
		image = gtk_image_new_from_pixbuf (NULL);
		gtk_container_add (GTK_CONTAINER(item->event), image);
		gtk_box_pack_start (GTK_BOX(cartoon->images_box), item->event,
				    TRUE, TRUE, 0);
		g_signal_connect (G_OBJECT(item->event), "button-press-event",
				  G_CALLBACK(button_press_cb), item);
		gtk_widget_show_all (item->event);

		item->buf = g_new(char, BUF_SIZE);
		item->loader = gdk_pixbuf_loader_new ();
		g_signal_connect (G_OBJECT(item->loader), "area_prepared",
				  G_CALLBACK(progressive_prepared_cb), image);
		g_signal_connect (G_OBJECT(item->loader), "area_updated",
				  G_CALLBACK(progressive_update_cb), image);

		gnome_vfs_async_open(&item->handle, item->url,
				     GNOME_VFS_OPEN_READ, -5,
				     (GnomeVFSAsyncOpenCallback)open_cb, item);
		list = list->next;
	}
	return;
}
